@@ -94,28 +94,59 @@ def bind(*args, without_prefix=False, positional=False):
             "See https://github.com/pseeth/argbind/tree/main/examples/hello_world#argbind-with-positional-arguments")
         patterns = []
 
-    def decorator(func):
-        PARSE_FUNCS[func.__name__] = (func, patterns, without_prefix, positional)
+
+    def decorator(object_or_func):
+        func = object_or_func
+        is_class = inspect.isclass(func)
+        if is_class:
+            func = getattr(func, "__init__")            
+
+        prefix = func.__qualname__
+        if "__init__" in prefix:
+            prefix = prefix.split(".")[0]
+        
+        # Check if function is bound already. If it is, just re-wrap it,
+        # instead of wrapping the function twice.
+        if prefix in PARSE_FUNCS:
+            func = PARSE_FUNCS[prefix][0]
+        else:
+            PARSE_FUNCS[prefix] = (func, patterns, without_prefix, positional)
         
         @wraps(func)
         def cmd_func(*args, **kwargs):
-            prefix = func.__name__
-            sig = inspect.signature(func)
+            parameters = list(inspect.signature(func).parameters.items())
+            
             cmd_kwargs = {}
+            pos_kwargs = {parameters[i][0]: arg for i, arg in enumerate(args)}
 
-            for key, val in sig.parameters.items():
-                arg_type = val.annotation
+            for key, val in parameters:
                 arg_val = val.default
                 if arg_val is not inspect.Parameter.empty or positional:
                     arg_name = f'{prefix}.{key}' if not without_prefix else f'{key}'
                     if arg_name in ARGS and key not in kwargs:
-                        cmd_kwargs[key] = ARGS[arg_name]
+                        val = ARGS[arg_name]
+                        if key in pos_kwargs:
+                            val = pos_kwargs[key]
+                        cmd_kwargs[key] = val
                         use_key = arg_name
                         if PATTERN:
                             use_key = f'{PATTERN}/{use_key}'
-                        USED_ARGS[use_key] = ARGS[arg_name]
+                        USED_ARGS[use_key] = val
 
             kwargs.update(cmd_kwargs)
+            cmd_args = []
+            for i, arg in enumerate(args):
+                key = parameters[i][0]
+                if key not in kwargs:
+                    cmd_args.append(arg)
+
+            # Ensure dictionary order is in parameter order
+            ordered_kwargs = {}
+            for k, _ in parameters:
+                if k in kwargs:
+                    ordered_kwargs[k] = kwargs[k]
+            kwargs = ordered_kwargs
+
             if 'args.debug' not in ARGS: ARGS['args.debug'] = False
             if ARGS['args.debug'] or DEBUG:
                 if PATTERN: 
@@ -123,7 +154,12 @@ def bind(*args, without_prefix=False, positional=False):
                 else:
                     scope = None
                 print(_format_func_debug(prefix, kwargs, scope))
-            return func(*args, **kwargs)
+            return func(*cmd_args, **kwargs)
+        
+        if is_class:
+            setattr(object_or_func, "__init__", cmd_func)
+            cmd_func = object_or_func
+
         return cmd_func
 
     if bound_fn_or_cls is None:
@@ -157,7 +193,7 @@ class bind_module:
         """
         for fn_name in dir(module):
             fn = getattr(module, fn_name)
-            if not isinstance(fn, type(sys)) and hasattr(fn, "__name__"):
+            if not isinstance(fn, type(sys)) and hasattr(fn, "__qualname__"):
                 if filter_fn(fn):
                     bound_fn = bind(fn, *scopes, **kwargs)
                     setattr(self, fn_name, bound_fn)
@@ -191,21 +227,22 @@ def dump_args(args, output_path):
             output.append(line)
         f.write('\n'.join(output))
 
-def load_args(input_path):
+def load_args(input_path_or_stream):
     """
-    Loads arguments from a given input path. If $include key is in
-    the args, you can include other y
+    Loads arguments from a given input path or file stream, if
+    the file is already open.
     """
-    with open(input_path, 'r') as f:
-        data = yaml.load(f, Loader=yaml.Loader)
+    if isinstance(input_path_or_stream, (str, Path)):
+        with open(input_path_or_stream, 'r') as f:
+            data = yaml.load(f, Loader=yaml.Loader)
+    else:
+        data = yaml.load(input_path_or_stream, Loader=yaml.Loader)
     
     if '$include' in data:
         include_files = data.pop('$include')
         include_args = {}
         for include_file in include_files:
-            with open(include_file, 'r') as f:
-                _include_args = yaml.load(f, Loader=yaml.Loader)
-            include_args.update(_include_args)
+            include_args.update(load_args(include_file))
         include_args.update(data)
         data = include_args
 
@@ -278,12 +315,13 @@ class str_to_dict():
 
         return _values
 
-def parse_args():
-    """
-    Goes through all detected functions that are
-    bound and adds them to the argument parser,
-    along with their scopes. Then parses the
-    command line and returns a dictionary.
+def build_parser():
+    """Builds the argument parser from all of the bound functions.
+
+    Returns
+    -------
+    ArgumentParser
+        Argument parser built by ArgBind.
     """
     p = argparse.ArgumentParser(
         formatter_class=argparse.RawTextHelpFormatter
@@ -297,10 +335,9 @@ def parse_args():
         help="Print arguments as they are passed to each function.")
 
     # Add kwargs from function to parser
-    for func_name in PARSE_FUNCS:
-        func, patterns, without_prefix, positional = PARSE_FUNCS[func_name]
+    for prefix in PARSE_FUNCS:
+        func, patterns, without_prefix, positional = PARSE_FUNCS[prefix]
         sig = inspect.signature(func)
-        prefix = func.__name__
 
         docstring = docstring_parser.parse(func.__doc__)
         parameter_help = docstring.params
@@ -393,6 +430,14 @@ def parse_args():
         desc = textwrap.fill(desc, width=HELP_WIDTH)
         f.description = desc
     
+    return p
+
+def parse_args(p=None):
+    """
+    Parses the command line and returns a dictionary.
+    Builds the argument parser if p is None.
+    """
+    p = build_parser() if p is None else p
     used_args = [x.replace('--', '').split('=')[0] for x in sys.argv if x.startswith('--')]
     used_args.extend(['args.save', 'args.load'])
 
